@@ -2,12 +2,14 @@
 
 namespace boru\ocr;
 
+use boru\ocr\Agent\AiOptions;
 use boru\ocr\Agent\AgentOptions;
 use boru\ocr\Agent\AiOcrOrchestrator;
+use boru\ocr\Layout\LayoutOptions;
+use boru\ocr\OcrEngineResult;
 use boru\ocr\Pipeline\OcrPipeline;
 use boru\ocr\Pipeline\OcrPipelineResultBundle;
-use boru\ocr\Pipeline\OcrResult;
-use boru\ocr\Format\FrameFormatter;
+use boru\ocr\Traits\OcrLogTrait;
 
 class OcrEngine
 {
@@ -29,14 +31,12 @@ class OcrEngine
     /** @var array */
     protected $providerOptions = [];
 
-    /** @var array */
-    protected $layoutOptions = [
-        'minConf' => 0,
-        'multiColumn' => true,
-    ];
+    /** @var LayoutOptions */
+    protected $layoutOptions;
 
-    /** @var callable|null */
+    /** @var OCRLogger|null */
     protected $logger;
+    use OcrLogTrait;
 
     /** @var string */
     protected $tesseractBinary = 'tesseract';
@@ -49,6 +49,11 @@ class OcrEngine
     protected $tableInterpreterOptions = null;
     protected $finalAgentOptions = null;
 
+    /** @var AiOptions|null */
+    protected $aiOptions = null;
+
+    
+
     /**
      * @param string $sourceFile
      */
@@ -98,20 +103,24 @@ class OcrEngine
     }
 
     /**
-     * @param array $opts
+     * @param array|LayoutOptions $opts
      * @return $this
      */
-    public function withLayoutOptions(array $opts)
+    public function withLayoutOptions($opts)
     {
-        $this->layoutOptions = $opts;
+        if ($opts instanceof LayoutOptions) {
+            $this->layoutOptions = $opts;
+        } elseif (is_array($opts)) {
+            $this->layoutOptions = LayoutOptions::create($opts);
+        }
         return $this;
     }
 
     /**
-     * @param callable $logger function(string $msg):void
+     * @param OCRLogger $logger
      * @return $this
      */
-    public function withLogger($logger)
+    public function withLogger(OCRLogger $logger)
     {
         $this->logger = $logger;
         return $this;
@@ -139,6 +148,15 @@ class OcrEngine
             $this->useAI = true;
             return $this;
         }
+        if($useAI instanceof AiOptions){
+            $this->aiOptions = $useAI;
+            $this->useAI = true;
+            return $this;
+        } elseif(is_array($useAI)) {
+            $this->aiOptions = AiOptions::fromArray($useAI);
+            $this->useAI = true;
+            return $this;
+        }
         $this->useAI = $useAI;
         return $this;
     }
@@ -185,12 +203,21 @@ class OcrEngine
      */
     public function run()
     {
+        if(!$this->logger){
+            $this->logger = new OCRLogger();
+        }
         if(!$this->imageDir){
             $this->imageDir = sys_get_temp_dir().'/ocr_'.uniqid();
             if (!is_dir($this->imageDir)) {
                @mkdir($this->imageDir, 0777, true);
             }
         }
+        if(!isset($this->tesseractOptions["logger"]) || $this->tesseractOptions["logger"] === null) {
+            $this->tesseractOptions["logger"] = $this->logger;
+        }
+        //if($this->useAI) {
+            //$this->layoutOptions["tableRegionMarkers"] = true;
+        //}
         $pipeline = new OcrPipeline(array(
             'logger' => $this->logger,
             'tesseractBinary' => $this->tesseractBinary,
@@ -203,7 +230,14 @@ class OcrEngine
         /** @var OcrPipelineResultBundle $bundle */
         $bundle = $pipeline->run($this->sourceFile);
         $result = new OcrEngineResult($bundle->result, $bundle->evidence);
-        if($this->useAI) {
+
+        // Skip AI for certain direct-text source types.
+        $sourceType = isset($bundle->result->metrics['type']) ? $bundle->result->metrics['type'] : null;
+        $skipAiTypes = array('spreadsheet', 'word');
+
+        $skipAi = in_array($sourceType, $skipAiTypes, true);
+
+        if($this->useAI && !$skipAi) {
             if(!$this->finalAgentOptions){
                 $this->finalAgentOptions = $this->defaultAgentOptions();
             }
@@ -213,13 +247,16 @@ class OcrEngine
             if(!$this->tableInterpreterOptions){
                 $this->tableInterpreterOptions = $this->defaultAgentOptions();
             }
-            $orc = new AiOcrOrchestrator([
-                "usePlanner" => $this->userPlanner,
-                "useTableInterpreter" => $this->useTableInterpreter,
-                "tableInterpreterOptions" => $this->tableInterpreterOptions,
-                "plannerAgentOptions" => $this->plannerAgentOptions,
-                "finalAgentOptions" => $this->finalAgentOptions,
-            ], $this->logger);
+            if(!$this->aiOptions){
+                $this->aiOptions = AiOptions::fromArray([
+                    "usePlanner" => $this->userPlanner,
+                    "useTableInterpreter" => $this->useTableInterpreter,
+                    "tableInterpreterOptions" => $this->tableInterpreterOptions,
+                    "plannerAgentOptions" => $this->plannerAgentOptions,
+                    "finalAgentOptions" => $this->finalAgentOptions,
+                ]);
+            }
+            $orc = new AiOcrOrchestrator($this->aiOptions, $this->logger);
 
             $final = $orc->run($this->sourceFile, $result);
             $result->setFinalText($final);
